﻿using CtrEditor.Convertidores;
using CtrEditor.Siemens;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using CtrEditor.Simulacion;

namespace CtrEditor.ObjetosSim
{
    /// <summary>
    /// Interaction logic for ucVMmotorSim.xaml
    /// </summary>
    /// 
    public class VMSimMotor
    {
        public bool _STATUS_VFD_Ready;
        public float STATUS_VFD_ACT_Speed_Hz;
        public bool Motor_Running;
        public bool STATUS_VFD_Trip;
        public bool STATUS_VFD_Warning;
        public bool STATUS_VFD_Coasting;
        public bool OUT_Run;
        public bool OUT_Stop;
        public bool OUT_Reversal;
        public float OUT_OUT_VFD_REQ_Speed_Hz;
    }


    public class osVMmotorSim : osBase
    {


        // Otros datos y métodos relevantes para la simulación

        private string _nombre = "VetroMeccanica Motor";
        private float _tamano;
        private float _left;
        private float _top;
        private int _numeroMotor;
        private float _ratio;
        private float _velocidad;
        private bool _encendido;
        public VMSimMotor motState = new VMSimMotor();


        public float Tamano
        {
            get => _tamano;
            set
            {
                _tamano = value;
                OnPropertyChanged(nameof(Tamano));
            }
        }

        public bool Encendido
        {
            get => _encendido;
            set
            {
                _encendido = value;
                OnPropertyChanged(nameof(Encendido));
            }
        }

        public int PLC_NumeroMotor
        {
            get => _numeroMotor;
            set
            {
                _numeroMotor = value;
                OnPropertyChanged(nameof(PLC_NumeroMotor));
            }
        }


        public override float Left
        {
            get => _left;
            set
            {
                _left = value;
                CanvasSetLeftinMeter(value);
                OnPropertyChanged(nameof(Left));
            }
        }
        public override float Top
        {
            get => _top;
            set
            {
                _top = value;
                CanvasSetTopinMeter(value);
                OnPropertyChanged(nameof(Top));
            }
        }

        public override string Nombre
        {
            get => _nombre;
            set
            {
                if (_nombre != value)
                {
                    _nombre = value;
                    OnPropertyChanged(nameof(Nombre));
                }
            }
        }

        public float Ratio {
            get => _ratio;
            set {
                _ratio = value;
                OnPropertyChanged(nameof(Ratio));
            }
        }

        public float Velocidad {
            get => _velocidad;
            set {
                _velocidad = value; 
                OnPropertyChanged(nameof(Velocidad));
            }
        }

        public osVMmotorSim()
        {
            Tamano = 0.30f;
            PLC_NumeroMotor = 31;
        }

        public override void UpdateGeometryStart()
        {
            // Se llama antes de la simulacion

        }
        public override void UpdateGeometryStep()
        {
        }
        public override void UpdatePLC(PLCModel plc) {
            var index = 0;
            switch (PLC_NumeroMotor)
            {
                case < 100: 
                    index = (int)PLC_NumeroMotor-30+300;
                    break;
            }

            motState.OUT_Run = plc.LeerTagBool($"\"DB MotorSimulate\".Motors[{index}].OUT.Run");
            motState.OUT_Reversal = plc.LeerTagBool($"\"DB MotorSimulate\".Motors[{index}].OUT.\"Reversal Direction\"");
            motState.OUT_OUT_VFD_REQ_Speed_Hz = (float)plc.LeerTagInt16($"\"DB MotorSimulate\".Motors[{index}].OUT.OUT_VFD_REQ_Speed_Hz");

            if (Encendido)
            {
                motState._STATUS_VFD_Ready = true;
                motState.Motor_Running = true;
                motState.STATUS_VFD_Trip = false;
                motState.STATUS_VFD_Warning = false;
                motState.STATUS_VFD_Coasting = false;
                if (motState.STATUS_VFD_ACT_Speed_Hz < motState.OUT_OUT_VFD_REQ_Speed_Hz)
                    motState.STATUS_VFD_ACT_Speed_Hz += motState.OUT_OUT_VFD_REQ_Speed_Hz / 375 ; // Simulate Ramp
            } else
            {
                motState._STATUS_VFD_Ready = false;
                motState.Motor_Running = false;
                motState.STATUS_VFD_Trip = true;
                motState.STATUS_VFD_Warning = false;
                motState.STATUS_VFD_Coasting = false;
                if (motState.STATUS_VFD_ACT_Speed_Hz > 0)
                    motState.STATUS_VFD_ACT_Speed_Hz -= 1; // Simulate Ramp
            }

            plc.EscribirTagBool($"\"DB MotorSimulate\".Motors[{index}].STATUS_VFD_Ready", motState._STATUS_VFD_Ready);
            plc.EscribirTagBool($"\"DB MotorSimulate\".Motors[{index}].Motor_Running", motState.Motor_Running);
            plc.EscribirTagBool($"\"DB MotorSimulate\".Motors[{index}].STATUS_VFD_Trip", motState.STATUS_VFD_Trip);
            plc.EscribirTagBool($"\"DB MotorSimulate\".Motors[{index}].STATUS_VFD_Warning", motState.STATUS_VFD_Warning);
            plc.EscribirTagBool($"\"DB MotorSimulate\".Motors[{index}].STATUS_VFD_Coasting", motState.STATUS_VFD_Coasting);

            plc.EscribirTagInt16($"\"DB MotorSimulate\".Motors[{index}].STATUS_VFD_ACT_Speed_Hz",(int)motState.STATUS_VFD_ACT_Speed_Hz);

            Velocidad = motState.STATUS_VFD_ACT_Speed_Hz/10;
        }

        public override void UpdateControl()
        {

        }
        public override void ucLoaded()
        {
            // El UserControl ya se ha cargado y podemos obtener las coordenadas para 
            // crear el objeto de simulacion
        }

    }

    public partial class ucVMmotorSim : UserControl, IDataContainer
    {
        public osBase? Datos { get; set; }

        public ucVMmotorSim()
        {
            InitializeComponent();
            this.Loaded += OnLoaded;
        }
        private void OnLoaded(object sender, RoutedEventArgs e)
        {
            Datos?.ucLoaded();
        }
        public void Resize(float width, float height) { }
        public void Move(float LeftPixels, float TopPixels)
        {
            if (Datos != null)
            {
                Datos.Left = PixelToMeter.Instance.calc.PixelsToMeters(LeftPixels);
                Datos.Top = PixelToMeter.Instance.calc.PixelsToMeters(TopPixels);
            }
        }
        public void Rotate(float Angle) { }
        public void Highlight(bool State) { }
        public int ZIndex()
        {
            return 10;
        }
    }
}
